import os
import json
from subprocess import *
from rcGlobalEnv import rcEnv

pathlib = os.path.dirname(__file__)
pathsvc = os.path.join(pathlib, '..')
nodeconf = os.path.join(pathsvc, "etc", "node.conf")

if rcEnv.sysname == "Windows":
    nodemgr = os.path.join(pathsvc, "nodemgr.cmd")
    svcmon = os.path.join(pathsvc, "svcmon.cmd")
    cron = os.path.join(pathsvc, "cron.cmd")
else:
    pathbin = os.path.join(pathsvc, "bin")
    nodemgr = os.path.join(pathbin, "nodemgr")
    svcmon = os.path.join(pathbin, "svcmon")
    cron = os.path.join(pathbin, "cron",  "cron")

import sys

sys.path.append(os.path.join(pathlib, 'cherrypy'))

import cherrypy

try:
    import ConfigParser
except ImportError:
    import configparser as ConfigParser

try:
    config = ConfigParser.RawConfigParser()
    config.read(nodeconf)
except:
    config = None

try:
    port = config.getint("listener", "port")
except:
    port = rcEnv.listener_port

try:
    ssl_certificate = config.get("listener", "ssl_certificate")
except:
    ssl_certificate = rcEnv.ssl_certificate

try:
    ssl_private_key = config.get("listener", "ssl_private_key")
except:
    ssl_private_key = rcEnv.ssl_private_key

try:
    password = config.get("listener", "password")
except:
    import crypt
    import uuid
    password = crypt.crypt(str(uuid.uuid1()),str(uuid.uuid1()))
    if not config.has_section("listener"):
        config.add_section("listener")
    config.set("listener", "password", password)
    try:
        fp = open(nodeconf, 'w')
        config.write(fp)
        fp.close()
    except:
        raise Exception("failed to set a new password. refuse to start without a password set in listener section of node.conf")

cherry_config = {
    'server.socket_host': '0.0.0.0',
    'server.socket_port': port,
    'server.ssl_module': 'pyopenssl',
    'server.ssl_certificate': ssl_certificate,
    'server.ssl_private_key': ssl_private_key,
}

#from md5 import md5
def encrypt_pw(pw):
    #return md5(pw).hexdigest()
    return pw

cherry_mnt_opts_pub = {
    'request.dispatch': cherrypy.dispatch.MethodDispatcher(),
}

cherry_mnt_opts_priv = {
    'request.dispatch': cherrypy.dispatch.MethodDispatcher(),
    'tools.basic_auth.on': True,
    'tools.basic_auth.realm': 'OpenSVC agent on '+rcEnv.nodename,
    'tools.basic_auth.users': {'opensvc': password},
    'tools.basic_auth.encrypt': encrypt_pw
}

cherrypy.config.update(cherry_config)


class NodeHostMode():
    exposed = True
    def GET(self, id=None):
        try:
            config.read(nodeconf)
            host_mode = config.get("node", "host_mode")
        except:
            host_mode = 'TST'
        return json.dumps({'status': 0, 'data': {'host_mode': host_mode}})

class NodeDequeueActions():
    exposed = True
    def GET(self, id=None):
        cmd = [nodemgr, 'dequeue_actions']
        p = Popen(cmd, stdout=PIPE, stderr=PIPE, stdin=None)
        out, err = p.communicate()
        return json.dumps({'status': p.returncode, 'stdout': out, 'stderr': err})

class Api(object):
    exposed = True

class listener(object):
    def __init__(self):
        cherrypy.tree.mount(
            NodeDequeueActions(), '/api/node/dequeue_actions',
            {'/': cherry_mnt_opts_pub}
        )
        cherrypy.tree.mount(
            NodeHostMode(), '/api/node/host_mode',
            {'/': cherry_mnt_opts_pub}
        )

        cherrypy.engine.start()

if __name__ == '__main__':
    a = listener()
    cherrypy.engine.block()

